@@ -1,7 +1,7 @@
 import pandas as pd
 
 from celescope.tools.step import Step, s_common
-from celescope.tools.plotly_plot import Tsne_dropdown_plot,Tsne_plot
+from celescope.tools.plotly_plot import Tsne_dropdown_plot,Tsne_single_plot,Tsne_plot
 
 
 
@@ -19,6 +19,7 @@ def analysis_cite(args):
 class Analysis_cite(Step):
     def __init__(self, args, display_title):
         super().__init__(args, display_title)
+        self.tmp_dir = self.args.outdir
 
         # input_file
         self.tsne_coord = self.args.tsne_coord
@@ -27,8 +28,15 @@ class Analysis_cite(Step):
         df_tsne = pd.read_csv(self.tsne_coord,sep="\t",index_col=0)
         feature_name_list = df_tsne.columns[4:].to_list()
 
+        # accelerate
+        df_tsne['tSNE_1'] = df_tsne['tSNE_1'].astype('float16')
+        df_tsne['tSNE_2'] = df_tsne['tSNE_2'].astype('float16')
+        for col in feature_name_list:
+            df_tsne[col] = df_tsne[col].astype('float16')
+
         # plot
         tsne_cluster = Tsne_plot(df_tsne, 'cluster').get_plotly_div()
         self.add_data(tsne_cluster=tsne_cluster)
         tsne_citeseq = Tsne_dropdown_plot(df_tsne,'Citeseq',feature_name_list).get_plotly_div()
         self.add_data(tsne_citeseq=tsne_citeseq)
+        Tsne_single_plot(df_tsne,feature_name_list,self.tmp_dir).get_plotly_div()
