@@ -1,3 +1,4 @@
+   
 import math
 
 import matplotlib.pyplot as plt
@@ -14,21 +15,18 @@ class Otsu():
     remove all zero in array
     Return 1 if len(array) < otsu_min_len
     """
-    def __init__(self, array, log_base=10, otsu_min_len = 50, otsu_plot_path=None,):
+    def __init__(self, array, log_base=10, otsu_min_len = 50, otsu_plot_path=None, **kwargs):
         
         self.len_bool = True
         array = [x for x in array if x > 0 ]
         if len(array) < otsu_min_len:
             self.len_bool = False
 
-        self.log_base = log_base
-        if log_base == 2:
-            self.array = np.log2(array)
-        elif log_base == 10:
-            self.array = np.log10(array)
-        else:
-            raise Exception('log_base must be 2, 10')
+        self.log_base = int(log_base)
+        # base change rule
+        self.array = np.log(array) / np.log(self.log_base)
 
+        self.kwargs = kwargs
         self.threshold = 1
         self.counts = None
         self.bins = None
@@ -37,7 +35,6 @@ class Otsu():
 
     def _threshold_otsu(self):
         """Return threshold value based on Otsu's method.
-
         hist : array, or 2-tuple of arrays, optional
             Histogram from which to determine the threshold, and optionally a
             corresponding array of bin center intensities.
@@ -100,20 +97,41 @@ class Otsu():
 
 class Auto():
     """
-    threshold = top 1% cell count / coef
+    threshold = top {percentile}% cell count / coef
+    count is usually UMI count.
+    >>> array = [50] * 100 + [30] * 100 + [10] * 100 + [4] * 100
+    >>> Auto(array, coef=10).run()
+    5
+    >>> Auto(array, percentile=70, coef=3).run()
+    10
+    >>> Auto(array, percentile=50, coef=10, expected_cell_num=100).run()
+    5
+    >>> Auto([1, 2, 20, 30, 40], expected_cell_num=4, percentile=50, coef=10).run()
+    2
     """
-    def __init__(self, array, coef=3):
+    def __init__(self, array, percentile=99, coef=3, expected_cell_num=None, **kwargs):
         self.array = [x for x in array if x > 0 ]
-        self.coef = coef
+        self.percentile = percentile
+        self.coef = int(coef)
+        self.expected_cell_num = expected_cell_num
+        self.kwargs = kwargs
     
     def run(self):
         array = self.array
         if not array:
             return 1
-        n_cell_1_percentile = len(array) // 100
+
+        if not self.expected_cell_num:
+            expected_cell_num = len(array)
+        else:
+            expected_cell_num = self.expected_cell_num
+            if expected_cell_num > len(array):
+                print('Warning: expected_cell_num > len(array)')
+                expected_cell_num = len(array)
+                      
         sorted_counts = sorted(array, reverse=True)
-        count_cell_1_percentile = sorted_counts[n_cell_1_percentile]
-        threshold = int(count_cell_1_percentile / self.coef)
+        count_cell_percentile = np.percentile(sorted_counts[:expected_cell_num], self.percentile)
+        threshold = int(count_cell_percentile / self.coef)
 
         return threshold
 
@@ -144,7 +162,7 @@ class Threshold():
             otsu = Otsu(self.array, otsu_plot_path=self.otsu_plot_path, **self.kwargs)
             threshold = otsu.run()
         elif self.threshold_method == 'auto':
-            auto = Auto(self.array)
+            auto = Auto(self.array, **self.kwargs)
             threshold = auto.run()
         elif self.threshold_method == 'hard':
             if self.hard_threshold:
